//**********************************************************************
// BRS
// ===
//
// Application to shut down Windows
//
// Syntax brs [-0/1/2/3] [<machine name>]
//
//  -0 = exit or logoff
//  -1 = restart or logoff
//  -2 = exit or shutdown
//  -3 = reboot
//
// If no flag is given then a dialog box is displayed with buttons to
// shut down windows.
//**********************************************************************

#ifndef STRICT
#define STRICT
#endif

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "brsdlg.h"


//**********************************************************************
// Function prototypes
//**********************************************************************

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow);
BOOL CALLBACK BrsProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

#ifdef WIN32
BOOL ShutDownWindows(int ShutDown, char* Machine);
const char* GetLastErrorMessage(void);
#else
BOOL ShutDownWindows(int ShutDown);
#endif

int winprintf(char *Format, ...);


//**********************************************************************
// Entry point from Windows
//**********************************************************************

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{ int shutdown, i;
  char* p;
  char machine[256];
#ifndef WIN32
  DLGPROC lp_proc;
#endif

// Process the command line

  shutdown = -1;
  strcpy(machine, "");

  for (p = lpCmdLine; *p != '\0'; p++)
  { while (*p == ' ') p++;

// '-' indicates a flag

    if (*p == '-')
    { switch (*(p+1))
      { case '0':
        case '1':
        case '2':
        case '3':
          shutdown = (int) (*(p+1)) - '0';
          break;
      }

      while (*p != ' ' && *p != '\0') p++;
    }

// Any other character is the start of the machine name, no further args
// are processed following this.

    else
    { for (i = 0; *p != ' ' && *p != '\0'; i++, p++)
        machine[i] = *p;
      machine[i] = '\0';

      break;
    }
  }

// Check if a command line shutdown argument was given

  if (shutdown >= 0)
  {
#ifdef WIN32
    ShutDownWindows(shutdown, machine);
#else
    ShutDownWindows(shutdown);
#endif
  }

// Else prompt for the shutdown info

  else
  {
#ifdef WIN32
    DialogBoxParam(hInstance, "BrsBoxNT", NULL, (DLGPROC) BrsProc, (LPARAM) machine);
#else
    lp_proc = (DLGPROC) MakeProcInstance((FARPROC) BrsProc, hInstance);
    DialogBox(hInstance, "BrsBoxW3", NULL, lp_proc);
    FreeProcInstance((FARPROC) lp_proc);
#endif
  }

// All finished

  return(0);
}


//**********************************************************************
// Message handler for dialog box
//**********************************************************************

BOOL CALLBACK BrsProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
#ifdef WIN32
  BOOL b;
  char machine[256];
#endif

  switch (message)
  {

// Initialise the dialog

    case WM_INITDIALOG:

// Shutdown can be restricted by settings in brs.ini

      if (GetPrivateProfileInt("Brs", "DisableLogoff", 0, "brs.ini") == 1)
        EnableWindow(GetDlgItem(hDlg, IDD_BRS_LOGOFF), FALSE);
      if (GetPrivateProfileInt("Brs", "DisableReboot", 0, "brs.ini") == 1)
        EnableWindow(GetDlgItem(hDlg, IDD_BRS_REBOOT), FALSE);
#ifdef WIN32
      if (GetPrivateProfileInt("Brs", "DisableShutdown", 0, "brs.ini") == 1)
        EnableWindow(GetDlgItem(hDlg, IDD_BRS_SHUTDOWN), FALSE);
#else
      if (GetPrivateProfileInt("Brs", "DisableRestart", 0, "brs.ini") == 1)
        EnableWindow(GetDlgItem(hDlg, IDD_BRS_RESTART), FALSE);
#endif

// For NT set the machine name

#ifdef WIN32
      SetDlgItemText(hDlg, IDD_BRS_MACHINE, (char*) lParam);
#endif
      return(TRUE);

// Commands

    case WM_COMMAND:
      switch (LOWORD(wParam))
      {

// Cancel pressed, just abort

        case IDCANCEL:
          EndDialog(hDlg, TRUE);
          return(TRUE);

// On any of the shutdown commands get the machine name

#ifdef WIN32
        case IDD_BRS_LOGOFF:
        case IDD_BRS_RESTART:
        case IDD_BRS_SHUTDOWN:
        case IDD_BRS_REBOOT:
          GetDlgItemText(hDlg, IDD_BRS_MACHINE, machine, 255);

          if (LOWORD(wParam) == IDD_BRS_LOGOFF)
          { b = ShutDownWindows(0, NULL);
          }
          else if (LOWORD(wParam) == IDD_BRS_RESTART)
          { b = ShutDownWindows(1, NULL);
          }
          else if (LOWORD(wParam) == IDD_BRS_SHUTDOWN)
          { b = ShutDownWindows(2, strlen(machine) > 0 ? machine : NULL);
          }
          else if (LOWORD(wParam) == IDD_BRS_REBOOT)
          { b = ShutDownWindows(3, strlen(machine) > 0 ? machine : NULL);
          }

          if (!b)
            winprintf("Error shutting down %s\r\n%s", machine, GetLastErrorMessage());

          EndDialog(hDlg, TRUE);
          return(TRUE);

#else
        case IDD_BRS_LOGOFF:
          ShutDownWindows(0);
          EndDialog(hDlg, TRUE);
          return(TRUE);

        case IDD_BRS_RESTART:
          ShutDownWindows(1);
          EndDialog(hDlg, TRUE);
          return(TRUE);

        case IDD_BRS_SHUTDOWN:
          ShutDownWindows(2);
          EndDialog(hDlg, TRUE);
          return(TRUE);

        case IDD_BRS_REBOOT:
          ShutDownWindows(3);
          EndDialog(hDlg, TRUE);
          return(TRUE);
#endif
      }
      break;
  }

  return(FALSE);
}


//**********************************************************************
// FUNCTION: ShutDownWindows
//
// ARGUMENTS: Shutdown - 0 = exit or logoff
//                       1 = restart or logoff
//                       2 = exit or shutdown
//                       3 = reboot
//
// RETURNS: FALSE if unsuccessful, TRUE if successful
//
// DESCRIPTION:
//   This function shuts down Windows in a variety of ways
//**********************************************************************

#ifdef WIN32
BOOL ShutDownWindows(int Shutdown, char* Machine)
#else
BOOL ShutDownWindows(int Shutdown)
#endif
{

#ifdef WIN32

  HANDLE token;
  TOKEN_PRIVILEGES tkp;
  BOOL result;

  switch (Shutdown)
  { case 2:
    case 3:
      if (!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &token))
      { result = FALSE;
        break;
      }

      LookupPrivilegeValue(NULL, "SeShutdownPrivilege", &tkp.Privileges[0].Luid);
      tkp.PrivilegeCount = 1;
      tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
      if (!AdjustTokenPrivileges(token, FALSE, &tkp, 0, NULL, 0))
      { result = FALSE;
        break;
      }

      if (Machine)
        result = InitiateSystemShutdown(Machine, NULL, 0, FALSE, Shutdown == 2 ? FALSE : TRUE);
      else
        result = ExitWindowsEx(Shutdown == 2 ? EWX_SHUTDOWN : EWX_REBOOT, 0);

      tkp.Privileges[0].Attributes = 0;
      AdjustTokenPrivileges(token, FALSE, &tkp, 0, NULL, 0);
      break;

    default:
      result = ExitWindowsEx(EWX_LOGOFF, 0);
      break;
  }

  if (!result)
  { switch (Shutdown)
    { case 2:
        result = ExitWindowsEx(EWX_SHUTDOWN, 0);

      case 3:
        result = ExitWindowsEx(EWX_REBOOT, 0);

      default:
        result = ExitWindowsEx(EWX_LOGOFF, 0);
    }
  }

  return(result);

#else

  switch (Shutdown)
  { case 1:
      ExitWindows(EW_RESTARTWINDOWS, 0);
      break;

    case 3:
      ExitWindows(EW_REBOOTSYSTEM, 0);
      break;

    default:
      ExitWindows(0, 0);
      break;
  }

  return(FALSE);

#endif
}


//**********************************************************************
// GetLastErrorMessage
// ===================
// Format an error message
//**********************************************************************

#ifdef WIN32
const char* GetLastErrorMessage(void)
{ DWORD d;
  static char errmsg[512];

  d = GetLastError();

  strcpy(errmsg, "<unknown error>");
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, d, 0, errmsg, 511, NULL);
  return(errmsg);
}
#endif


//**********************************************************************
// FUNCTION: winprintf
//
// ARGUMENTS: Format - printf format string
//            ...    - vararg list
//
// RETURNS: Always returns IDOK
//
// DESCRIPTION:
//   Windows version of printf, the output is displayed in a message
//   box.
//   To use this function you must #include <stdio.h> and <stdarg.h>.
//**********************************************************************

int winprintf(char *Format, ...)
{ char s[256];
  va_list ap;

  va_start(ap, Format);
  vsprintf(s, Format, ap);
  va_end(ap);

  return(MessageBox(GetFocus(), s, "BRS", MB_OK | MB_ICONEXCLAMATION));
}


